/*****************************************************************************
 *      Symmetricom, Inc
 *      bc635/736PCI-V2/PCIe Time & Frequency Processor
 *      Rev 8.0 Mar 28, 2009  SY
 *      Rev 7.0 Mar 28, 2008  SY
 *      Rev 4.4 Nov 06, 2000  SD
 *      Copyright (c) Symmetricom, Inc.
 ****************************************************************************/
/*****************************************************************************
 * bc635pci.c is a C test program that demonstrates the operation of the different
 * functions of the bc635PCI Time & Frequency Module using the stfp device driver.
 * The code is written in such a way that the user can cut and paste parts of the
 * program to their own custom application.
****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <errno.h>
#include <termio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <signal.h>
#include "stfpio.h"


//****************************************************************************
// Global variables and forward declaration
//****************************************************************************
int fd,tm_fmt;
struct getdata_t get;
char rbuf[STFP_MAX_READ];
char datbuf[STFP_MAX_READ];
u_char req41[5] = { 0x32, 10, 0x41, 1, 0x21};

struct {			
	u_char	id;		
	float	time;		
	short	week;	
	float	offset;	
} pac41;

struct {			
	u_char	id;		
	float	X;		
	float	Y;	
	float	Z;
	float	TofF;
} pac42;

int menu_main ();
void getPac41 ();
void printPac41 ();
void getPac42 ();
void printPac42 ();
void getPac46 ();

u_char getByte ();
short  getShort ();
float  getFloat ();
double getDouble ();
int kbhit (void);
int WaitForKbhit(int,int);
void usererr ();
void errout ();
void errin();
void nxt ();

//****************************************************************************
// Newly defined API functions
//****************************************************************************
boolean_t bcReqRevisionID (int fd, int* id);
boolean_t bcSetPeriodicDDSSelect (int fd, int sel);
boolean_t bcSetPeriodicDDSEnable (int fd, int enable);
boolean_t bcSetDDSDivider (int fd, int divider);
boolean_t bcSetDDSDividerSource (int fd, int src);
boolean_t bcSetDDSSyncMode (int fd, int mode);
boolean_t bcSetDDSMultiplier (int fd, int mult);
boolean_t bcSetDDSPeriodValue (int fd, uint32_t period);
boolean_t bcSetDDSTuningWord (int fd, uint32_t tuneWord);
boolean_t bcSetDDSFrequency (int fd, double fFreq);
boolean_t bcSetTcInEx (int fd, uchar_t TcIn, uchar_t SubType);
boolean_t bcSetGenCodeEx (int fd, uchar_t GenTc, uchar_t SubType);
boolean_t bcReqEventsData (int fd, struct EventsData *pDat);
boolean_t bcSetEventsData (int fd, struct EventsData *pDat);


//****************************************************************************
// Description : main
//****************************************************************************
int main (int argc, char **argv)
{
    if ( (fd = open ("/dev/stfp0", O_RDWR)) < 0 ){  /* Open device */
        printf("Error opening Device Driver ......... Exiting");
        _exit(1);
    }

    get.arg = GETDATA_TFORMAT;                      /* Time Format */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        tm_fmt = (int)get.data.tformat;

    menu_main ();
    close(fd);
}

//****************************************************************************
// Description : Read Time in Binary/Decimal
//****************************************************************************
void pci_read_time ()
{
    struct stfp_time stm;			
    struct stfp_timeval tvTime;	
    int i;

    printf ("\n\n\n\n");
    if (tm_fmt == 0){   /* Decimal Time Format*/
        do {
            if (!WaitForKbhit(0, 100)){
                ioctl (fd, RDTIMEREQ, &stm);				
                printf (" Julian Time: %03d %d %02d:%02d:%02d.%06d%d   Status: %x\n",
                    stm.tm.tm_yday+1, stm.tm.tm_year+1900,
                    stm.tm.tm_hour, stm.tm.tm_min, stm.tm.tm_sec, stm.usec, stm.hnsec, stm.status);
            }
        }while(!kbhit());
    }
    else {              /* Binary Time Format */						
        do {
            if (!WaitForKbhit(0, 100)){
                ioctl (fd, RDTIMETVREQ, &tvTime);
                printf ("Binary Time: %lu.%06ld%d status: %x     ", 
                    tvTime.tv.tv_sec, tvTime.tv.tv_usec, tvTime.hnsec, tvTime.status);
                printf ("%s", ctime ((time_t *)&tvTime.tv.tv_sec));
            }
        }while(!kbhit());
    }
    fgetc (stdin);
    fflush (stdin);
}

//****************************************************************************
// Description : Read Event Time
//****************************************************************************
void pci_read_event_time()
{		
    struct stfp_timeval tvTime;	
    unsigned long evtmaj, evtmin;

    printf ("\n\n"); /* Binary Time Format */
    do {
        ioctl (fd, RDEVENTTV, &tvTime);
        if ( tvTime.tv.tv_sec != evtmaj || tvTime.tv.tv_usec != evtmin){
            printf ("Event Time: %lu.%06ld%d status: %x     ", 
                tvTime.tv.tv_sec, tvTime.tv.tv_usec, tvTime.hnsec, tvTime.status);
            printf ("%s", ctime ((time_t *)&tvTime.tv.tv_sec));
        }
        evtmaj = tvTime.tv.tv_sec;
        evtmin = tvTime.tv.tv_usec;
    }while(!kbhit());

    fgetc (stdin);
    fflush (stdin);
}

//****************************************************************************
// Description : Read Event2 Time
//****************************************************************************
void pci_read_event2_time()
{		
    struct stfp_timeval tvTime;	
    unsigned long evtmaj, evtmin;

    printf ("\n\n"); /* Binary Time Format */
    do {
        ioctl (fd, RDEVENT2TV, &tvTime);
        if ( tvTime.tv.tv_sec != evtmaj || tvTime.tv.tv_usec != evtmin){
            printf ("Event Time: %lu.%06ld%d status: %x     ", 
                tvTime.tv.tv_sec, tvTime.tv.tv_usec, tvTime.hnsec, tvTime.status);
            printf ("%s", ctime ((time_t *)&tvTime.tv.tv_sec));
        }
        evtmaj = tvTime.tv.tv_sec;
        evtmin = tvTime.tv.tv_usec;
    }while(!kbhit());

    fgetc (stdin);
    fflush (stdin);
}

//****************************************************************************
// Description : Read Event3 Time
//****************************************************************************
void pci_read_event3_time()
{		
    struct stfp_timeval tvTime;	
    unsigned long evtmaj, evtmin;

    printf ("\n\n"); /* Binary Time Format */
    do {
        ioctl (fd, RDEVENT3TV, &tvTime);
        if ( tvTime.tv.tv_sec != evtmaj || tvTime.tv.tv_usec != evtmin){
            printf ("Event Time: %lu.%06ld%d status: %x     ", 
                tvTime.tv.tv_sec, tvTime.tv.tv_usec, tvTime.hnsec, tvTime.status);
            printf ("%s", ctime ((time_t *)&tvTime.tv.tv_sec));
        }
        evtmaj = tvTime.tv.tv_sec;
        evtmin = tvTime.tv.tv_usec;
    }while(!kbhit());

    fgetc (stdin);
    fflush (stdin);
}

//****************************************************************************
// Description : Set Strobe Time
//****************************************************************************
void pci_set_strobe ()
{
    int strbmode;
    struct stfp_time stm;
    struct stfp_timeval tvTime;
    unsigned long tm_sec, tm_usec;
    int styear, stmon, stday, sthour, stmin, stsec;

    printf ("\n\nSelect Strobe Mode:\n\n");
    printf (" 1. Major and Minor\n");
    printf (" 2. Minor Only\n");
    printf (" 0. Cancel\n\n");
    printf ("    Select: ");
    scanf  ("%d", &strbmode);

    if (strbmode == 1){
        ioctl (fd, CONTROLSTROBE, STROBE_SECUS);

        printf("\n\nEnter Major Time:\n\nYYYY MM DD HH MM SS\n");
        scanf("%d %d %d %d %d %d", &styear, &stmon, &stday,
                &sthour, &stmin, &stsec);

        stm.tm.tm_year  = styear - 1900;
        stm.tm.tm_mon   = stmon - 1;
        stm.tm.tm_mday  = stday;
        stm.tm.tm_hour  = sthour;
        stm.tm.tm_min   = stmin;
        stm.tm.tm_sec   = stsec;
        tm_sec = mktime ((struct tm *)&stm);
        tvTime.tv.tv_sec = tm_sec;
        printf("\nEnter Minor time: ");
        scanf("%lu", &tm_sec);
        tvTime.tv.tv_usec =tm_usec;
        if (ioctl (fd, WRSTROBETV, &tvTime)<0)
            errin ();
    }
    else if (strbmode == 2){
        ioctl (fd, CONTROLSTROBE, STROBE_USONLY);
        tvTime.tv.tv_sec = 0;
        printf("\nEnter minor time: ");
        scanf("%lu", &tm_usec);
        tvTime.tv.tv_usec =tm_usec;
        if (ioctl (fd, WRSTROBETV, &tvTime)<0)
            errin ();
    }
}

//****************************************************************************
// Description: Set Control Register
//****************************************************************************
void pci_set_control()
{
    unsigned long ctlreg;

    if (ioctl(fd, RDCONTROL, &ctlreg)<0)
        errout();
    ctlreg &= 0xffffffff;
    printf ("\n\nCurrent Control Register Value: 0x%08lx", ctlreg);
    printf ("\nEnter New CR Value (four Hex): ");
    scanf  ("%lx", &ctlreg);
    if (ioctl(fd, SETCONTROL, ctlreg)<0)
        errin();
}

//****************************************************************************
// Description: Set Timing Mode
//****************************************************************************
void pci_mode()
{
    int oper_mode = 0;

    printf ("\n\nSelect Operational Mode:\n\n");
    printf (" 0. Time Code Mode\n");
    printf (" 1. Free Running Mode\n");
    printf (" 2. External 1PPS\n");
    printf (" 3. Real Time Clock Mode\n");
    printf (" 4. GPS Mode\n\n");
    printf ("    Select: ");
    scanf  ("%d", &oper_mode);

    switch (oper_mode)
    {
        case 0: ioctl (fd, SELTIMINGMODE, MODE_TIMECODE);   /* TimeCode Mode */
            break;
        case 1: ioctl (fd, SELTIMINGMODE, MODE_FREERUN);    /* Free Running Mode */
            break;
        case 2: ioctl (fd, SELTIMINGMODE, MODE_EXT1PPS);    /* 1PPS Mode */
            break;
        case 3: ioctl (fd, SELTIMINGMODE, MODE_RTC);        /* RTC Mode */
            break;
        case 4: ioctl (fd, SELTIMINGMODE, MODE_GPS);        /* GPS Mode */
            break;
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Set Time Code Format & Type
//****************************************************************************
void pci_time_code ()
{
    int tc_fmt;
    int tc_mod;

    printf ("\n\nSelect Time Code Format:\n\n");
    printf (" 1. IRIG A\n");
    printf (" 2. IRIG B\n");
    printf (" 3. IEEE 1344\n");
    printf (" 4. NASA 36\n\n");
    printf ("    Select: ");
    scanf  ("%d", &tc_fmt);
    switch ( tc_fmt )
    {
        case 1: ioctl (fd, SELTCFORMAT, TC_IRIGA);  /* IRIG A time code */
            break;
        case 2: ioctl (fd, SELTCFORMAT, TC_IRIGB);  /* IRIG B time code */
            break;
        case 3: ioctl (fd, SELTCFORMAT, TC_IEEE);   /* IEEE time code */
            break;
        case 4: ioctl (fd, SELTCFORMAT, TC_NASA);   /* NASA 36 time code */
            break;
        default:usererr();
            break;
    }

    printf ("\n\nSelect Time Code Modulation:\n\n");
    printf (" 1. Modulated\n");
    printf (" 2. DC Level Shift\n\n");
    printf ("    Select: ");
    scanf  ("%d", &tc_mod);
    switch ( tc_mod )
    {
        case 1: ioctl (fd, SELTCMOD, MOD_AM);       /* Modulated */
            break;
        case 2: ioctl (fd, SELTCMOD, MOD_DCLS);     /* DCLS */
            break;
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Set Output Frequency
//****************************************************************************
void pci_out_freq ()
{
    int out_freq = 3;
    printf ("\n\nSelect Output Frequency:\n\n");
    printf (" 1. 1MHz\n 2. 5MHz\n 3. 10MHz\n\n");
    printf (" Select: ");
    scanf  ("%d", &out_freq);
    switch ( out_freq )
    {
        case 1: ioctl (fd, SELFREQUENCYOUT, FREQ_1MHZ);     /* 1 MHz */
            break;
        case 2: ioctl (fd, SELFREQUENCYOUT, FREQ_5MHZ);     /* 5 MHz */
            break;
        case 3: ioctl (fd, SELFREQUENCYOUT, FREQ_10MHZ);    /* 10 MHz */
            break;
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Set Time Format
//****************************************************************************
void pci_time_format()
{
    int time_format = 0;
	
    printf ("\n\nSelect Time Format\n  0. Decimal\n  1. Binary\n\nSelect: ");
    scanf ("%d", &time_format);
	
    switch ( time_format )
    {
        case 0: ioctl (fd, SELTIMEFORMAT, TIME_DECIMAL);    /* decimal time format */
            tm_fmt = 0;
            break;
        case 1: ioctl (fd, SELTIMEFORMAT, TIME_BINARY);     /* Binary time format */
            tm_fmt = 1;
            break;				
        default:usererr();
            break;
    }
}
	
//****************************************************************************
// Description : Set Periodic (Heartbeat) Output
//****************************************************************************
void pci_heartbeat ()
{
    struct periodic sper;
    int hrt_mode;
    int counter1 = 10; 
    int counter2 = 10;
    sper.sync1pps = PERIODIC_SYNC;

    printf ("\n\nHeartbeat Counters and Mode:\n\n");
    printf (" 1. Synchronous mode\n");
    printf (" 2. Asynchronous mode\n\n");
    printf (" Select: ");
    scanf  ("%i", &hrt_mode);
    printf ("\n Enter decimal value for counter1: ");
    scanf  ("%d", &counter1);
    printf ("\n Enter decimal value for counter2: ");
    scanf  ("%d", &counter2);

    if (hrt_mode == 2)
        sper.sync1pps = PERIODIC_NOSYNC;
    if (counter1 >= 2 && counter1 <= 65535)
        sper.n1 = counter1;
    if (counter2 >= 2 && counter2 <= 65535)
        sper.n2 = counter2;

    if (ioctl (fd, SETPERIODIC, &sper) < 0)
    {
        printf("\nError setting Heartbeat Counters!!!");
        nxt();
    }
}

//****************************************************************************
// Description : Set Year
//****************************************************************************
void pci_set_year ()
{
    int year = 2000;

    printf ("\n\nEnter Year (1970 to 2050): ");
    scanf  ("%d", &year);
    if (year > 1970 && year < 2050){
        if (ioctl (fd, SETYEAR, year) < 0){     /* Set year */
            errin();
            nxt();
        }
    }
    else
        usererr();
}

//****************************************************************************
// Description : Set Time
//****************************************************************************
void pci_set_time ()
{
    struct stfp_time stm;
    struct stfp_timeval tvTime;
    unsigned long tm_sec;
    struct stfp_dec_tm dec;
    int styear, stmon, stday, sthour, stmin, stsec;

    /* Decimal Time Format */
    if (tm_fmt == 0)
    {
        printf("\n\nEnter Time:\n\nYYYY DDD HH MM SS\n");
        scanf("%d %d %d %d %d", &styear, &stday,
                &sthour, &stmin, &stsec);

        dec.dec_year    = styear;
        dec.dec_day     = stday;
        dec.dec_hour    = sthour;
        dec.dec_min     = stmin;
        dec.dec_sec     = stsec;

        if ( ioctl (fd, SETDECTIME, dec) < 0 )
            printf("\nError Setting Time!!");
    }
    else    /* Binary time format */
    {
        printf("\n\nEnter Time:\n\nYYYY MM DD HH MM SS\n");
        scanf("%d %d %d %d %d %d", &styear, &stmon, &stday,
                &sthour, &stmin, &stsec);

        stm.tm.tm_year  = styear - 1900;
        stm.tm.tm_mon   = stmon - 1;
        stm.tm.tm_mday  = stday;
        stm.tm.tm_hour  = sthour;
        stm.tm.tm_min   = stmin;
        stm.tm.tm_sec   = stsec;
        tm_sec = mktime ((struct tm*)&stm);

        if ( ioctl (fd, SETTIME, tm_sec) < 0 )
            printf("\nError Setting Time!!");
    }
}

//****************************************************************************
// Description : Set Propogation Delay
//****************************************************************************
void pci_set_prop_delay ()
{
    long prop_delay = 0;

    printf("\n\nEnter Propogation Delay: ");
    scanf("%ld", &prop_delay);

    if ( ioctl (fd, SETTIMINGOFFSET, prop_delay) < 0 )
    {
        errin();
        nxt();
    }
}

//****************************************************************************
// Description : Software Rest
//****************************************************************************
void pci_sw_rest ()
{
    int yesno;

    printf("\n\nReset the board?\n 0. No\n 1. Yes\n\nSelect: ");
    scanf("%d", &yesno);

    if ( yesno == 1 )
    {
        if (ioctl (fd, SOFTWARERESET, 1) < 0)
        {
            printf("\nError reseting the card!!");
            nxt();
        }
    }
}

//****************************************************************************
// Description : Set TimeCode Output Format
//****************************************************************************
void pci_tc_out_format()
{
    int tc_out = 0;
	
    printf ("\n\nSelect Time Code Output Format\n  0. IRIG B\n  1. IEEE 1344\n\nSelect: ");
    scanf ("%d", &tc_out);
	
    switch ( tc_out )
    {
        case 0: ioctl (fd, SETTCOUTFMT, TC_IRIGB);	
            break;
        case 1: ioctl (fd, SETTCOUTFMT, TC_IEEE);		
            break;				
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Set Clock Source
//****************************************************************************
void pci_set_clock_src()
{
    int clk_src = 0;
	
    printf ("\n\nSelect Clock Source\n  0. Internal\n  1. External\n\nSelect: ");
    scanf ("%d", &clk_src);

    switch ( clk_src )
    {
        case 0: ioctl (fd, CONTROLTIMEBASE, CLOCK_INTERNAL);	
            break;
        case 1: ioctl (fd, CONTROLTIMEBASE, CLOCK_EXTERNAL);		
            break;				
        default: usererr();
            break;
    }
}

//****************************************************************************
// Description : Set Generator Time Offset
//****************************************************************************
void pci_set_gen_off ()
{
    struct tcgenoffset gen;
    short gentc;			
    int gentcflg;

    printf ("\n\nEnter Generator Time Offset (-16 to 16): ");
    scanf  ("%hd", &gentc);

    printf ("\nAdd Half Hour?\n 0: No\n 1: Yes\nSelect: ");
    scanf ("%d", &gentcflg);

    gen.genoff = gentc;
    gen.genflg = (u_char)gentcflg;
	
    if ( ioctl (fd, SETGENTMOFFSET, &gen) < 0 ){
        printf("\nError setting generator time offset");
        nxt();
    }
}

//****************************************************************************
// Description : Set Local Time Offset
//****************************************************************************
void pci_set_loc_off ()
{
	struct	loctmoffset loc;
	short	localoff;			
	int		localflg;

	printf ("\n\nEnter Local Time Offset (-16 to 16): ");
	scanf  ("%hd", &localoff);

	printf ("\nAdd Half Hour?\n 0: No\n 1: Yes\nSelect: ");
	scanf ("%d", &localflg);

	loc.locoff = localoff;
	loc.locflg = (u_char)localflg;
	
	if ( ioctl (fd, SETLOCTMOFFSET, &loc) < 0 ){
		printf("\nError setting local time offset");
		nxt();
	}
}

//****************************************************************************
// Description : Program Leap Seconds
//****************************************************************************
void pci_set_leap_sec ()
{
    struct leapseconds leap;
    long   leaptm;
    int    leapflag;

    printf ("\n\nEnter Leap Event Time in Seconds (Unix Time): ");
    scanf  ("%ld", &leaptm);

    printf ("\nLeap Second Flag?\n  0: Disable\n  1: Insertion\n -1: Deletion\nSelect: ");
    scanf ("%d", &leapflag);

    leap.leap_tm = leaptm;
    leap.ls_flag = (u_char)leapflag;

    if ( ioctl (fd, SETLEAPSECEVENT, &leap) < 0 ){
        printf("\nError setting leap event time");
        nxt();
    }
}
       
//****************************************************************************
// Description : Control Jam-Sync
//****************************************************************************
void pci_ctl_jam_sync()
{
    int jam_sync = 1;

    printf ("\n\nControl Jam-Sync\n 0. Disable Jam-Sync\n 1. Enable Jam-Sync\n\nSelect: ");
    scanf ("%d", &jam_sync);

    switch ( jam_sync )
    {
        case 0: ioctl (fd, CONTROLTIMEBASE, JAMSYNC_DISABLE);	
            break;
        case 1: ioctl (fd, CONTROLTIMEBASE, JAMSYNC_ENABLE);		
            break;				
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Force Jam-Sync
//****************************************************************************
void pci_frc_jam_sync()
{
    int jam_sync = 0;

    printf ("\n\nForce Jam-Sync?\n 0. No\n 1. Yes\n\nSelect: ");
    scanf ("%d", &jam_sync);

    if ( jam_sync ){
        ioctl (fd, CONTROLTIMEBASE, FORCE_JAMSYNC);	
        nxt();
    }
}

//****************************************************************************
// Description : Load D/A Converter
//****************************************************************************
void pci_set_da_con()
{
    u_short da_con = 0;
	
    printf ("\n\nEnter D/A value (0 to 65535): ");
    scanf ("%d", &da_con);
	
    if ( ioctl (fd, SETDAC, da_con) < 0 ){
        printf("\nError setting D/A converter!");		
        nxt();
    }
}

//****************************************************************************
// Description : Set Disciplining Gain
//****************************************************************************
void pci_set_gain()
{
    short dis_gain = 0;
	
    printf ("\n\nEnter Gain value (-32768 to 32767): ");
    scanf ("%hd", &dis_gain);
	
    if ( ioctl (fd, SETDISCGAIN, dis_gain) < 0 ){
        printf("\nError setting disciplining gain");		
        nxt();
    }
}

//****************************************************************************
// Description : Sync RTC to External Time Data
//****************************************************************************
void pci_sync_rtc()
{
    int sync_rtc = 0;

    printf ("\n\nSync RTC to External Time Data?\n 0. No\n 1. Yes\n\nSelect: ");
    scanf ("%d", &sync_rtc);
	
    if ( sync_rtc ){
        if ( ioctl (fd, SYNCRTC) < 0 ){
            errin();
            nxt();
        }
    }
}

//****************************************************************************
// Description : Disconnect Battery to RTC Chip
//****************************************************************************
void pci_dis_rtc()
{
    int dis_rtc = 0;
	
    printf ("\n\nDisconnect Battery to RTC Chip?\n 0. No\n 1. Yes\n\nSelect: ");
    scanf ("%d", &dis_rtc);
	
    if ( dis_rtc ){
        if ( ioctl (fd, DISCBATT) < 0 ){
            errin();
            nxt();
        }
    }
}

//****************************************************************************
// Description : Set Clock Value
//****************************************************************************
void pci_set_clk_val()
{
    long clk_val = 0;
	
    printf ("\n\nEnter Clock Value: ");
    scanf ("%ld", &clk_val);
	
    if ( ioctl (fd, SETCLKVAL, clk_val) < 0 ){
        printf("\nError setting clock value");
        nxt();
    }
}

//****************************************************************************
// Description : Set GPS Time Format
//****************************************************************************
void pci_set_gps_tm_fmt()
{
    int gps_tm_fmt = 0;
	
    printf ("\n\nGPS Time Format\n 0. UTC Time\n 1. GPS Time\n\nSelect: ");
    scanf ("%d", &gps_tm_fmt);
	
    switch ( gps_tm_fmt )
    {
        case 0: ioctl (fd, SETGPSTMFMT, UTC_FMT);	
            break;
        case 1: ioctl (fd, SETGPSTMFMT, GPS_FMT);		
            break;				
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Set GPS Mode Flag
//****************************************************************************
void pci_set_gps_mode_flg()
{
    int gps_mode_flg = 1;
	
    printf ("\n\nGPS Mode Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
    scanf ("%d", &gps_mode_flg);
	
    switch ( gps_mode_flg )
    {
        case 0: ioctl (fd, SETGPSTMFMT, GPS_FLG_DIS);	
            break;
        case 1: ioctl (fd, SETGPSTMFMT, GPS_FLG_ENA);		
            break;				
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Set Local Time Offset Flag
//****************************************************************************
void pci_set_local_off_flg()
{
    int local_off_flg = 1;
	
    printf ("\n\nLocal Time Offset Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
    scanf ("%d", &local_off_flg);

    switch ( local_off_flg )
    {
        case 0: ioctl (fd, SETLOCTMFLG, LOC_OFF_DIS);	
            break;
        case 1: ioctl (fd, SETLOCTMFLG, LOC_OFF_ENA);		
            break;				
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Set Year Auto Increment Flag
//****************************************************************************
void pci_set_yr_auto_inc_flg()
{
    int yr_auto_flg = 1;
	
    printf ("\n\nYear Auto Increment Flag\n 0. Disable\n 1. Enable\n\nSelect: ");
    scanf ("%d", &yr_auto_flg);
	
    switch ( yr_auto_flg )
    {
        case 0: ioctl (fd, SETYRINCFLG, YR_INC_DIS);	
            break;
        case 1: ioctl (fd, SETYRINCFLG, YR_INC_ENA);		
            break;				
        default:usererr();
            break;
    }
}

//****************************************************************************
// Description : Request Time Settings
//****************************************************************************
void pci_req_time_settings ()
{
    const char smode[7][10] = {"Time Code","Free Run","Ext 1PPS","RTC","Reserved","Reserved","GPS"};
    const char stm_fmt[2][10] = {"Decimal","Binary"};
    const char tcout_IRIGA[8][36] = {
        "IRIG A000, A130 (BCD,CF,SBS)", "IRIG A001, A131 (BCD,CF)",
        "IRIG A002, A132 (BCD)", "IRIG A003, A133 (BCD,SBS)",
        "IRIG A004, A134 (BCD,YR,CF,SBS)", "IRIG A005, A135 (BCD,YR,CF)",
        "IRIG A006, A136 (BCD,YR)", "IRIG A007, A137 (BCD,YR,SBS)",
    };
    const char tcout_IRIGB[8][36] = {
        "IRIG B000, B120 (BCD,CF,SBS)", "IRIG B001, B121 (BCD,CF)",
        "IRIG B002, B122 (BCD)", "IRIG B003, B123 (BCD,SBS)",
        "IRIG B004, B124 (BCD,YR,CF,SBS)", "IRIG B005, B125 (BCD,YR,CF)",
        "IRIG B006, B126 (BCD,YR)", "IRIG B007, B127 (BCD,YR,SBS)",
    };
    u_char tc_fmt[2], tc_mod, mode, tc_out[2];
    int year;

    get.arg = GETDATA_MODE;         /* Timing Mode */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        mode = (int)get.data.tmode;

    get.arg = GETDATA_TFORMAT;      /* Time Format */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        tm_fmt = (int)get.data.tformat;

    get.arg = GETDATA_YEAR ;        /* Year */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        year = (int)get.data.year;

    get.arg = GETDATA_TCFMT;        /* Time Code Format */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        tc_fmt[0] = get.data.tcformat[0];
        tc_fmt[1] = get.data.tcformat[1];
    }

    get.arg = GETDATA_TCMOD;        /* Time Code Modulation */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        tc_mod = (u_char)get.data.tcmod;

    get.arg = GETDATA_TCOUTFMT;     /* Time Code Output Format */
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        tc_out[0] = (u_char)get.data.tcoutfmt[0];
        tc_out[1] = (u_char)get.data.tcoutfmt[1];
    }

    printf("\n\nTime Settings:\n");
    printf("\n Mode\t\t\t: %s",smode[mode]);
    printf("\n Time Format\t\t: %s",stm_fmt[tm_fmt]);
    printf("\n Year\t\t\t: %d",year);

    switch (tc_fmt[0])
    {
        case TC_IRIGA:
            if (tc_fmt[1] == TC_SUBTYPE_NONE)
                printf("\n Input Time Code\t: IRIG A000-A003, A130-A133 (no year)" );
            else if (tc_fmt[1] == TC_SUBTYPE_Y)
                printf("\n Input Time Code\t: IRIG A004-A007, A134-A137 (has year)" );
            else
                printf("\n Input Time Code\t: Unknown" );
            break;
        case TC_IRIGB:
            if (tc_fmt[1] == TC_SUBTYPE_NONE)
                printf("\n Input Time Code\t: IRIG B000-B003, B120-B123 (no year)" );
            else if (tc_fmt[1] == TC_SUBTYPE_Y)
                printf("\n Input Time Code\t: IRIG B004-B007, B124-B127 (has year)" );
            else if (tc_fmt[1] == TC_SUBTYPE_T)
                printf("\n Input Time Code\t: IRIG B TrueTime Legacy" );
            else
                printf("\n Input Time Code\t: Unknown" );
            break;
        case TC_IEEE:
            printf("\n Input Time Code\t: IRIG B IEEE 1344" );
            break;
        case TC_IRIGE:
            if (tc_fmt[1] == TC_SUBTYPE_NONE)
                printf("\n Input Time Code\t: IRIG E001, E002, E121, E122 (no year)" );
            else if (tc_fmt[1] == TC_SUBTYPE_Y)
                printf("\n Input Time Code\t: IRIG E005, E006, E125, E126 (has year)" );
            else
                printf("\n Input Time Code\t: Unknown" );
            break;
        case TC_IRIGe:
            if (tc_fmt[1] == TC_SUBTYPE_NONE)
                printf("\n Input Time Code\t: IRIG E001, E002, E111, E112 (no year)" );
            else if (tc_fmt[1] == TC_SUBTYPE_Y)
                printf("\n Input Time Code\t: IRIG E005, E006, E115, E116 (has year)" );
            else
                printf("\n Input Time Code\t: Unknown" );
            break;
        case TC_IRIGG:
            if (tc_fmt[1] == TC_SUBTYPE_NONE)
                printf("\n Input Time Code\t: IRIG G001, G002, G141, G142 (no year)" );
            else if (tc_fmt[1] == TC_SUBTYPE_Y)
                printf("\n Input Time Code\t: IRIG G005, G006, G145, E146 (has year)" );
            else
                printf("\n Input Time Code\t: Unknown" );
            break;
        case TC_NASA:
            printf("\n Input Time Code\t: NASA 36" );
            break;
        case TC_XR3:
            printf("\n Input Time Code\t: XR3" );
            break;
        case TC_2137:
            printf("\n Input Time Code\t: 2137" );
            break;
        default:
            printf("\n Input Time Code\t: Unknown" );
            break;
    }

    switch ( tc_mod )
    {
        case MOD_AM:
            printf("\n Code Modulation\t: AM");
            break;
        case MOD_DCLS:
            printf("\n Code Modulation\t: DC");
            break;
        default:
            printf("\n Code Modulation\t: Unknown");
            break;
    }

    switch ( tc_out[0] )
    {
        case TC_IRIGA:
            if (tc_out[1] >= TC_SUBTYPE_0 && tc_out[1] <= TC_SUBTYPE_7)
                printf("\n Output Time Code\t: %s", tcout_IRIGA[tc_out[1]-TC_SUBTYPE_0]);
            else
                printf("\n Output Time Code\t: Unknown");
            break;
        case TC_IRIGB:
            if (tc_out[1] >= TC_SUBTYPE_0 && tc_out[1] <= TC_SUBTYPE_7)
                printf("\n Output Time Code\t: %s", tcout_IRIGB[tc_out[1]-TC_SUBTYPE_0]);
            else if (tc_out[1] == TC_SUBTYPE_T)
                printf("\n Output Time Code\t: IRIG B TrueTime (BCD,CF,SBS)");
            else
                printf("\n Output Time Code\t: Unknown");
            break;
        case TC_IEEE:
            printf("\n Output Time Code\t: IRIG B IEEE 1344");
            break;
        case TC_IRIGE:
            if (tc_out[1] == TC_SUBTYPE_1)
                printf("\n Output Time Code\t: IRIG E001, E121 (BCD,CF)");
            else if (tc_out[1] == TC_SUBTYPE_2)
                printf("\n Output Time Code\t: IRIG E002, E122 (BCD)");
            else if (tc_out[1] == TC_SUBTYPE_5)
                printf("\n Output Time Code\t: IRIG E005, E125 (BCD,YR,CF)");
            else if (tc_out[1] == TC_SUBTYPE_6)
                printf("\n Output Time Code\t: IRIG E006, E126 (BCD,YR)");
            else
                printf("\n Output Time Code\t: Unknown");
            break;
        case TC_IRIGe:
            if (tc_out[1] == TC_SUBTYPE_1)
                printf("\n Output Time Code\t: IRIG E001, E111 (BCD,CF)");
            else if (tc_out[1] == TC_SUBTYPE_2)
                printf("\n Output Time Code\t: IRIG E002, E112 (BCD)");
            else if (tc_out[1] == TC_SUBTYPE_5)
                printf("\n Output Time Code\t: IRIG E005, E115 (BCD,YR,CF)");
            else if (tc_out[1] == TC_SUBTYPE_6)
                printf("\n Output Time Code\t: IRIG E006, E116 (BCD,YR)");
            else
                printf("\n Output Time Code\t: Unknown");
            break;
        case TC_IRIGG:
            if (tc_out[1] == TC_SUBTYPE_5)
                printf("\n Output Time Code\t: IRIG G005, G145 (BCD,YR,CF)");
            else
                printf("\n Output Time Code\t: Unknown");
            break;
        case TC_NASA:
            printf("\n Output Time Code\t: NASA 36");
            break;
        case TC_XR3:
            printf("\n Output Time Code\t: XR3");
            break;
        case TC_2137:
            printf("\n Output Time Code\t: 2137");
            break;
        default:
            printf("\n Output Time Code\t: Unknown");
            break;
    }	

    nxt();
}

//****************************************************************************
// Description : Request Clock Settings
//****************************************************************************
void pci_req_clock_settings ()
{
    int dac_val, jam_ctl, bat_st, clk_val, dis_ctl, ph_ctl;
    int phase_st, dis_gain;
    const char jam[2][8]={"Disable","Enable"};
    const char batt[2][8]={"Failed","OK"};
    float filter;
    u_char clk_scr;

    get.arg = GETDATA_CLKSRC;       /* Clock Source */	
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        clk_scr = (u_char)get.data.clksrc;

    get.arg = GETDATA_DAVAL;        /* DAC Value */		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        dac_val = (int)get.data.davalue;

    get.arg = GETDATA_JAMSC;        /* Jam Control */	
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        jam_ctl = get.data.jsctl & 0x01;

    get.arg = GETDATA_BATTSTAT;     /* Batt Status */		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        bat_st = (get.data.battstat & 0x10) >> 4;

    get.arg = GETDATA_CLKVAL;       /* ClockValue */		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        clk_val = (int)get.data.clkval;

    get.arg = GETDATA_OSCCTL;       /* Oscillator Disciplining Control */		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        dis_ctl = get.data.ocsctl.discctl;
        ph_ctl = get.data.ocsctl.phasectl;
        phase_st = (int)get.data.ocsctl.phasestep;
        dis_gain = (int)get.data.ocsctl.gain;
        filter = (float)get.data.ocsctl.con;
    }

    printf("\n\nClock Settings:\n");

    if (clk_scr == 'E')
        printf("\n Oscillator  \t: External");
    else
        printf("\n Oscillator  \t: Internal");
	
    printf("\n DAC Value\t: %d",dac_val);
    printf("\n Jam Control\t: %s",jam[jam_ctl]);
    printf("\n Battery Status\t: %s",batt[bat_st]);
    printf("\n Clock Value\t: %d",clk_val);
    printf("\n Disc Control\t: %s",jam[dis_ctl]);
    printf("\n Phase Control\t: %s",jam[ph_ctl]);
    printf("\n Phase Step\t: %d",phase_st);
    printf("\n Disc Gain\t: %d",dis_gain);
    printf("\n Filter Const\t: %f",filter);

    nxt();
}

//****************************************************************************
// Description : Request Offsets Settings
//****************************************************************************
void pci_req_offsets_settings ()
{
    float loc_off, gen_off;
    int prop_off, loc_off_flg, gen_off_flg;
    short off_temp;
	
    get.arg = GETDATA_LOCTMOFF;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        off_temp = get.data.locoff.locoff;
	loc_off_flg = (int)get.data.locoff.locflg;
    }
	
    loc_off = (float)off_temp;
    if ( loc_off_flg )
        loc_off += 0.5;
	
    get.arg = GETDATA_TCGENOFF;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        off_temp = (float)get.data.genoff.genoff;
        gen_off_flg = (int)get.data.genoff.genflg;
    }
	
    gen_off = (float)off_temp;
    if ( gen_off_flg )
        gen_off += 0.5;
	
    get.arg = GETDATA_TOFF;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) ) 
        prop_off = (int)get.data.offset;

    printf("\n\nOffsets Settings:\n");
    printf("\n Local Offset     \t: %.1f",loc_off);
    printf("\n Generator Offset \t: %.1f",gen_off);
    printf("\n Propogation Delay\t: %d",prop_off);

    nxt();
}

//****************************************************************************
// Description : Request UTC Information
//****************************************************************************
void pci_req_utc_info ()
{
    int fmt, leap_sec, leap_sec_flg;
    const char gps_fmt[2][4]={"UTC","GPS"};
    unsigned int evttm;
	
    get.arg = GETDATA_UTCINFO;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        fmt = get.data.utc.enable & 0x01;
        leap_sec = get.data.utc.leapsec;
        leap_sec_flg = get.data.utc.leapevt;
        evttm = get.data.utc.evttime;
		
        printf("\n\nUTC Information\n");
        printf("\n GPS Time Format\t: %s\n Leap Seconds\t\t: %d", gps_fmt[fmt], leap_sec);

        switch (leap_sec_flg){
            case -1: printf("\n Leap Second Flag\t: Deletion");
                break;
            case 0:  printf("\n Leap Second Flag\t: Disable");
                break;
            case 1:  printf("\n Leap Second Flag\t: Addition");
                break;
            default: printf("\n Leap Second Flag\t: Unknown");
                break;
        }
	printf("\n Leap Event Time\t: %lu", evttm);
    }
    else
        printf("\nError retrieving data!!");

    nxt();
}

//****************************************************************************
// Description : Request Model, Serial Number, Assembly Number and Hardware FAB
//****************************************************************************
void pci_req_assembly ()
{
    u_char t1,t2,t3,t4,t5,t6,t7,t8;
    u_short assembly, hw_fab;
    unsigned int ser_num;

    printf("\n\nModel Information\n");

    get.arg = GETDATA_TFPMODEL;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        t1 = get.data.tfpm.tfp1;
        t2 = get.data.tfpm.tfp2;
        t3 = get.data.tfpm.tfp3;
        t4 = get.data.tfpm.tfp4;
        t5 = get.data.tfpm.tfp5;
        t6 = get.data.tfpm.tfp6;
        t7 = get.data.tfpm.tfp7;
        t8 = get.data.tfpm.tfp8;
        printf("\n Model\t\t: %c%c%c%c%c%c%c%c",
            t1,t2,t3,t4,t5,t6,t7,t8);
    }

    get.arg = GETDATA_SERIAL;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        ser_num = get.data.serialnum;
        printf("\n Serial Number\t: %08lx", ser_num);
    }

    get.arg = GETDATA_ASSEMB;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        assembly = (u_short)get.data.assemb;
        printf("\n Assembly Num\t: %hu", assembly);
    }

    get.arg = GETDATA_HWFAB;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        hw_fab = (u_short)get.data.hwfab;
        printf("\n Hardware FAB\t: %hu", hw_fab);
    }
    else
        printf("\nError retrieving data!!");

    nxt();
}

//****************************************************************************
// Description : Request Firmware Version
//****************************************************************************
void pci_req_fw_ver ()
{
    int fwmaj, fwmin, fwmon, fwday, fwyr;
    u_char fwb01, fwb02,fwb03,fwb04,fwb05,fwb06,fwb07,fwb08,fwb09,fwb10,fwb11;
 
    get.arg = GETDATA_DTFW;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        fwb01 = (u_char)get.data.dtfw.dt01;
        fwb02 = (u_char)get.data.dtfw.dt02;
        fwb03 = (u_char)get.data.dtfw.dt03;
        fwb04 = (u_char)get.data.dtfw.dt04;
        fwb05  = (u_char)get.data.dtfw.dt05;
        fwb06 = (u_char)get.data.dtfw.dt06;
        fwb07 = (u_char)get.data.dtfw.dt07;
        fwb08 = (u_char)get.data.dtfw.dt08;
        fwb09 = (u_char)get.data.dtfw.dt09;
        fwb10  = (u_char)get.data.dtfw.dt10;
        fwb11  = (u_char)get.data.dtfw.dt11;
        printf("\n\nFirmware Information\n");
        printf("\n\n Number\t\t: %c%c%c%c%c%c%c%c%c%c%c", 
        fwb01,fwb02,fwb03,fwb04,fwb05,fwb06,fwb07,fwb08,fwb09,fwb10,fwb11);
    }

    get.arg = GETDATA_FWVER;		
    if ( !((ioctl (fd, GETDATA, &get)) < 0) )
    {
        fwmaj = (u_char)get.data.version.vmajor;
        fwmin = (u_char)get.data.version.vminor;
        fwmon = (u_char)get.data.version.month;
        fwday = (u_char)get.data.version.day;
        fwyr  = (u_short)get.data.version.year;
		printf("\n Version\t: %d.%d\n Date\t\t: %d/%d/%d", 
        fwmaj, fwmin, fwmon,fwday,fwyr);
    }
    else
        printf("\nError retrieving firmware data!!");

    nxt();
}

//****************************************************************************
// Description : Interrupt handler
//****************************************************************************
void intr_handler (int sig)
{
    int32_t intsrc = 0;
    struct stfp_time stm;
    struct stfp_timeval tvTime;
    char buf[64];

    if (sig != SIGUSR1)
        return;

    signal (SIGUSR1, intr_handler);

    if ( ioctl (fd, RDINTSIGNAL, &intsrc) < 0 )
    {
        errout ();
        fgetc (stdin);
        fflush (stdin);
        return;
    }

    if (intsrc & INT_EVENT) {
        printf("Event | ");
    }

    if (intsrc & INT_PERIODIC) {
        printf("Periodic | ");
    }

    if (intsrc & INT_STROBE) {
        printf("Strobe | ");
    }

    if (intsrc & INT_1PPS) {
        printf("1PPS | ");
    }

    if (intsrc & INT_PACKET) {
        printf("GPS | ");
    }

    if (intsrc & INT_EVENT2) {
        printf("Event2 | ");
    }

    if (intsrc & INT_EVENT3) {
        printf("Event3 | ");
    }

    if (tm_fmt == 0) {   /* Decimal Time Format*/
        ioctl (fd, RDTIMEREQ, &stm);
        printf ("%02d:%02d:%02d.%06d%d   Status: %x",
             stm.tm.tm_hour, stm.tm.tm_min, stm.tm.tm_sec, stm.usec, stm.hnsec, stm.status);
    }
    else {              /* Binary Time Format */
        ioctl (fd, RDTIMETVREQ, &tvTime);
        ctime_r ((time_t *)&tvTime.tv.tv_sec, buf, 64);
        if (buf[strlen(buf) - 1] == '\n')
            buf[strlen(buf) - 1] = 0;
        printf ("%s   %lu.%06ld%d   Status: %x",
             buf, tvTime.tv.tv_sec, tvTime.tv.tv_usec, tvTime.hnsec, tvTime.status);
    }
    printf("\nPress 2 and Enter to stop interrupt routine!\n\n");

    fgetc (stdin);
    fflush (stdin);
}

//****************************************************************************
// Description : Set interrupt mask
//****************************************************************************
void pci_set_ints ()
{
    int cmd = 0;
    pthread_t thread;

    printf ("\n\n");
    printf ("Enter two lower case hex digits (example 0x02, 0x0a): 0x");
    scanf ("%x", &cmd);

    cmd &= 0x7F;

    signal (SIGUSR1, intr_handler);
    if ( ioctl (fd, SETINTSIGNAL, cmd) < 0 )
    {
        errout ();
    }
}

//****************************************************************************
// Description : Stop interrupts
//****************************************************************************
void pci_stop_ints ()
{
    uint32_t mask = 0;
    if ( ioctl (fd, SETINTSIGNAL, mask) < 0 )
    {
        errout ();
    }
}

//****************************************************************************
// Description : Stop interrupts
//****************************************************************************
void pci_req_ints ()
{
    int32_t mask = 0;
    if ( ioctl (fd, RDINTSIGNAL, &mask) < 0 )
    {
        errout ();
    }
    else
    {
        printf("\nInterrupt: 0x%08x", mask);
    }
}

//****************************************************************************
// Description : Request PCI Revision ID of the PCI configuration space
//****************************************************************************
void pci_req_rev_id ()
{
    int revID;

    if ( bcReqRevisionID (fd, &revID) == B_TRUE )
    {
        printf ("\n\n PCI Board Revision ID : %d (0x%2x)\n", revID, revID);
    }
    else
    {
        errout();
    }

    nxt();
}

//****************************************************************************
// Description : Read from dual port RAM
//****************************************************************************
void pci_read_dp_ram ()
{
    struct dp_rdwr_t rdwr;
    uint32_t iOffset;

    printf ("\n\n");
    printf ("Enter hex offset (0x000 - 0x800): 0x");
    scanf ("%x", &iOffset);
    rdwr.offset = (uint16_t)iOffset;

    if ( ioctl (fd, RDDPOFFSET, &rdwr) < 0 )
    {
        errout ();
    }
    else
    {
        printf ("Read DP RAM 0x%03x: 0x%02x\n",
                (uint32_t)rdwr.offset, (uint32_t)rdwr.data);
    }

    nxt();
}

//****************************************************************************
// Description : Write to dual port RAM
//****************************************************************************
void pci_write_dp_ram ()
{
    struct dp_rdwr_t rdwr;
    uint32_t iOffset, iData;

    printf ("\n\n");
    printf ("Enter hex offset (0x000 - 0x800): 0x");
    scanf ("%x", &iOffset);
    printf ("Enter hex data (0x00 - 0xFF): 0x");
    scanf ("%x", &iData);
    rdwr.offset = (uint16_t)iOffset;
    rdwr.data = (uchar_t)(iData & 0xFF);

    if ( ioctl (fd, WRDPOFFSET, &rdwr) < 0 )
    {
        errout ();
    }
    else
    {
        printf ("Wrote DP RAM 0x%03x with 0x%02x\n",
                (uint32_t)rdwr.offset, (uint32_t)rdwr.data);
    }

    nxt();
}

//****************************************************************************
//  Description : Write ACK register to send DP command to timing engine
//****************************************************************************
void pci_write_ack ()
{
    if ( ioctl (fd, SENDCOMMAND) < 0 )
    {
        errout ();
    }
    else
    {
        printf ("Wrote ACK register\n");
    }

    nxt();
}

//****************************************************************************
// Description : Select periodic output or DDS output
//****************************************************************************
void dds_set_periodic_dds_select ()
{
    int iVal;

    printf ("\n\n");
    printf ("Enter choice (0 - periodic output, 1 - DDS output): ");
    scanf ("%d", &iVal);

    if (bcSetPeriodicDDSSelect (fd, iVal) == B_TRUE)
    {
        printf ("Done setting periodic/DDS output\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Enable or disable periodic/DDS output
//****************************************************************************
void dds_set_periodic_dds_enable ()
{
    int iVal;

    printf ("\n\n");
    printf ("Enter choice (0 - disable DDS/periodic output, 1 - enable DDS/periodic output): ");
    scanf ("%d", &iVal);

    if (bcSetPeriodicDDSEnable (fd, iVal) == B_TRUE)
    {
        printf ("Done setting periodic/DDS enable or disable\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Set DDS divider value
//****************************************************************************
void dds_set_dds_divider ()
{
    int iVal;

    printf ("\n\n");
    printf ("Choice (0 - div by 10^0, 1 - div by 10^1, 2 - div by 10^2,\n");
    printf ("        3 - div by 10^3, 4 - div by 10^4, 5 - div by 10^5,\n");
    printf ("        6 - div by 10^6, 7 - div by 10^7,\n");
    printf ("        15 - div by period value in the period register)\n");
    printf ("Enter choice: ");
    scanf ("%d", &iVal);

    if (bcSetDDSDivider (fd, iVal) == B_TRUE)
    {
        printf ("Done setting DDS divider\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Set DDS divider source
//****************************************************************************
void dds_set_dds_divider_source ()
{
    int iVal;

    printf ("\n\n");
    printf ("Enter choice (0 - DDS, 1 - DDS multiplier, 2 - 100MHz): ");
    scanf ("%d", &iVal);

    if (bcSetDDSDividerSource (fd, iVal) == B_TRUE)
    {
        printf ("Done setting DDS divider source\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Set DDS sync mode
//****************************************************************************
void dds_set_dds_sync_mode ()
{
    int   iVal;

    printf ("\n\n");
    printf ("Enter choice (0 - fractional mode, 1 - continuous mode): ");
    scanf ("%d", &iVal);

    if (bcSetDDSSyncMode (fd, iVal) == B_TRUE)
    {
        printf ("Done setting DDS sync mode\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Set DDS multiplier
//****************************************************************************
void dds_set_dds_multiplier ()
{
    int   iVal;

    printf ("\n\n");
    printf ("Choice (1 - DDS x 1,   2 - DDS x 2,   3 - DDS x 3,\n");
    printf ("        4 - DDS x 4,   6 - DDS x 6,   8 - DDS x 8,\n");
    printf ("        10 - DDS x 10, 16 - DDS x 16)\n");
    printf ("Enter choice: ");
    scanf ("%d", &iVal);

    if (bcSetDDSMultiplier (fd, iVal) == B_TRUE)
    {
        printf ("Done setting DDS multiplier\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Set DDS period value
//****************************************************************************
void dds_set_dds_period_value ()
{
    uint32_t iVal;

    printf ("\n\n");
    printf ("Enter period value: ");
    scanf ("%d", &iVal);

    if (bcSetDDSPeriodValue (fd, iVal) == B_TRUE)
    {
        printf ("Done setting DDS period value\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Set DDS tuning word
//****************************************************************************
void dds_set_dds_tuning_word ()
{
    uint32_t iVal;

    printf ("\n\n");
    printf ("Enter tuning word: ");
    scanf ("%d", &iVal);

    if (bcSetDDSTuningWord (fd, iVal) == B_TRUE)
    {
        printf ("Done setting DDS tuning word\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Set DDS output frequency
//****************************************************************************
void dds_set_dds_frequency ()
{
    double fVal;

    printf ("\n\n");
    printf ("Enter DDS Frequency (in Hz): ");
    scanf ("%lf", &fVal);

    if (bcSetDDSFrequency (fd, fVal) == B_TRUE)
    {
        printf ("Done setting DDS frequency\n");
    }
    else
        errout();

    nxt();
}

//****************************************************************************
// Description : Set Time Code Format & Type Extended (V2 Hardware)
//****************************************************************************
void pci_time_code_ex ()
{
    int tc_fmt = 0;
    int tc_mod = 0;

    do {
        printf ("\n\nSelect Time Code Format:\n\n");
        printf ("  1. IRIG A000-A003, A130-A133 (no year)\n");
        printf ("  2. IRIG A004-A007, A134-A137 (has year)\n");
        printf ("  3. IRIG B000-B003, B120-B123 (no year)\n");
        printf ("  4. IRIG B004-B007, B124-B127 (has year)\n");
        printf ("  5. IRIG B TrueTime Legacy\n");
        printf ("  6. IRIG B IEEE 1344\n");
        printf ("  7. IRIG E001, E002, E121, E122 (no year)\n");
        printf ("  8. IRIG E005, E006, E125, E126 (has year)\n");
        printf ("  9. IRIG E001, E002, E111, E112 (no year)\n");
        printf (" 10. IRIG E005, E006, E115, E116 (has year)\n");
        printf (" 11. IRIG G001, G002, G141, G142 (no year)\n");
        printf (" 12. IRIG G005, G006, G145, G146 (has year)\n");
        printf (" 13. NASA 36\n");
        printf (" 14. XR3\n");
        printf (" 15. 2137\n");
        printf ("  0. Back to main menu \n\n");
        printf ("     Select: ");
        scanf  ("%d", &tc_fmt);
        switch ( tc_fmt )
        {
            case 1:
                if (bcSetTcInEx (fd, TC_IRIGA, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 2:
                if (bcSetTcInEx (fd, TC_IRIGA, TC_SUBTYPE_Y) == B_FALSE)
                    errout();
                break;
            case 3:
                if (bcSetTcInEx (fd, TC_IRIGB, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 4:
                if (bcSetTcInEx (fd, TC_IRIGB, TC_SUBTYPE_Y) == B_FALSE)
                    errout();
                break;
            case 5:
                if (bcSetTcInEx (fd, TC_IRIGB, TC_SUBTYPE_T) == B_FALSE)
                    errout();
                break;
            case 6:
                if (bcSetTcInEx (fd, TC_IEEE, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 7:
                if (bcSetTcInEx (fd, TC_IRIGE, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 8:
                if (bcSetTcInEx (fd, TC_IRIGE, TC_SUBTYPE_Y) == B_FALSE)
                    errout();
                break;
            case 9:
                if (bcSetTcInEx (fd, TC_IRIGe, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 10:
                if (bcSetTcInEx (fd, TC_IRIGe, TC_SUBTYPE_Y) == B_FALSE)
                    errout();
                break;
            case 11:
                if (bcSetTcInEx (fd, TC_IRIGG, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 12:
                if (bcSetTcInEx (fd, TC_IRIGG, TC_SUBTYPE_Y) == B_FALSE)
                    errout();
                break;
            case 13:
                if (bcSetTcInEx (fd, TC_NASA, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 14:
                if (bcSetTcInEx (fd, TC_XR3, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 15:
                if (bcSetTcInEx (fd, TC_2137, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 0:
            default:
                tc_fmt = 0;
                break;
        }

        if (tc_fmt == 0)
            break;

        printf ("\n\nSelect Time Code Modulation:\n\n");
        printf (" 1. Modulated\n");
        printf (" 2. DC Level Shift\n\n");
        printf ("    Select: ");
        scanf  ("%d", &tc_mod);
        switch ( tc_mod )
        {
            case 1: ioctl (fd, SELTCMOD, MOD_AM);       /* Modulated */
                break;
            case 2: ioctl (fd, SELTCMOD, MOD_DCLS);     /* DCLS */
                break;
            default:usererr();
                break;
        }
    } while (tc_fmt!=0);
}

//****************************************************************************
// Description : Set TimeCode Output Format Extended (V2 Hardware)
//****************************************************************************
void pci_tc_out_format_ex ()
{
    int tc_out = 0;

    do {
        printf ("\n\nSelect Time Code Output Format\n\n");
        printf ("  1. IRIG A000, A130 (BCD,CF,SBS)      2. IRIG A001, A131 (BCD,CF)\n");
        printf ("  3. IRIG A002, A132 (BCD)             4. IRIG A003, A133 (BCD,SBS)\n");
        printf ("  5. IRIG A004, A134 (BCD,YR,CF,SBS)   6. IRIG A005, A135 (BCD,YR,CF)\n");
        printf ("  7. IRIG A006, A136 (BCD,YR)          8. IRIG A007, A137 (BCD,YR,SBS)\n");
        printf ("\n");
        printf ("  9. IRIG B000, B120 (BCD,CF,SBS)     10. IRIG B001, B121 (BCD,CF)\n");
        printf (" 11. IRIG B002, B122 (BCD)            12. IRIG B003, B123 (BCD,SBS)\n");
        printf (" 13. IRIG B004, B124 (BCD,YR,CF,SBS)  14. IRIG B005, B125 (BCD,YR,CF)\n");
        printf (" 15. IRIG B006, B126 (BCD,YR)         16. IRIG B007, B127 (BCD,YR,SBS)\n");
        printf (" 17. IRIG B TrueTime (BCD,CF,SBS)     18. IRIG B IEEE 1344\n");
        printf ("\n");
        printf (" 19. IRIG E001, E121 (BCD,CF)         20. IRIG E002, E122 (BCD)\n");
        printf (" 21. IRIG E005, E125 (BCD,YR,CF)      22. IRIG E006, E126 (BCD,YR)\n");
        printf (" 23. IRIG E001, E111 (BCD,CF)         24. IRIG E002, E112 (BCD)\n");
        printf (" 25. IRIG E005, E115 (BCD,YR,CF)      26. IRIG E006, E116 (BCD,YR)\n");
        printf ("\n");
        printf (" 27. IRIG G005, G145 (BCD,YR,CF)      28. NASA 36\n");
        printf (" 29. XR 3                             30. 2137\n");
        printf ("  0. Back to main menu \n\n");
        printf ("     Select: ");
        scanf ("%d", &tc_out);

        switch ( tc_out )
        {
            case 1:
                if (bcSetGenCodeEx (fd, TC_IRIGA, TC_SUBTYPE_0) == B_FALSE)
                    errout();
                break;
            case 2:
                if (bcSetGenCodeEx (fd, TC_IRIGA, TC_SUBTYPE_1) == B_FALSE)
                    errout();
                break;
            case 3:
                if (bcSetGenCodeEx (fd, TC_IRIGA, TC_SUBTYPE_2) == B_FALSE)
                    errout();
                break;
            case 4:
                if (bcSetGenCodeEx (fd, TC_IRIGA, TC_SUBTYPE_3) == B_FALSE)
                    errout();
                break;
            case 5:
                if (bcSetGenCodeEx (fd, TC_IRIGA, TC_SUBTYPE_4) == B_FALSE)
                    errout();
                break;
            case 6:
                if (bcSetGenCodeEx (fd, TC_IRIGA, TC_SUBTYPE_5) == B_FALSE)
                    errout();
                break;
            case 7:
                if (bcSetGenCodeEx (fd, TC_IRIGA, TC_SUBTYPE_6) == B_FALSE)
                errout();
                break;
            case 8:
                if (bcSetGenCodeEx (fd, TC_IRIGA, TC_SUBTYPE_7) == B_FALSE)
                    errout();
                break;
            case 9:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_0) == B_FALSE)
                    errout();
                break;
            case 10:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_1) == B_FALSE)
                    errout();
                break;
            case 11:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_2) == B_FALSE)
                    errout();
                break;
            case 12:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_3) == B_FALSE)
                    errout();
                break;
            case 13:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_4) == B_FALSE)
                    errout();
                break;
            case 14:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_5) == B_FALSE)
                    errout();
                break;
            case 15:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_6) == B_FALSE)
                    errout();
                break;
            case 16:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_7) == B_FALSE)
                    errout();
                break;
            case 17:
                if (bcSetGenCodeEx (fd, TC_IRIGB, TC_SUBTYPE_T) == B_FALSE)
                    errout();
                break;
            case 18:
                if (bcSetGenCodeEx (fd, TC_IEEE, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 19:
                if (bcSetGenCodeEx (fd, TC_IRIGE, TC_SUBTYPE_1) == B_FALSE)
                    errout();
                break;
            case 20:
                if (bcSetGenCodeEx (fd, TC_IRIGE, TC_SUBTYPE_2) == B_FALSE)
                    errout();
                break;
            case 21:
                if (bcSetGenCodeEx (fd, TC_IRIGE, TC_SUBTYPE_5) == B_FALSE)
                    errout();
                break;
            case 22:
                if (bcSetGenCodeEx (fd, TC_IRIGE, TC_SUBTYPE_6) == B_FALSE)
                    errout();
                break;
            case 23:
                if (bcSetGenCodeEx (fd, TC_IRIGe, TC_SUBTYPE_1) == B_FALSE)
                    errout();
                break;
            case 24:
                if (bcSetGenCodeEx (fd, TC_IRIGe, TC_SUBTYPE_2) == B_FALSE)
                    errout();
                break;
            case 25:
                if (bcSetGenCodeEx (fd, TC_IRIGe, TC_SUBTYPE_5) == B_FALSE)
                    errout();
                break;
            case 26:
                if (bcSetGenCodeEx (fd, TC_IRIGe, TC_SUBTYPE_6) == B_FALSE)
                    errout();
                break;
            case 27:
                if (bcSetGenCodeEx (fd, TC_IRIGG, TC_SUBTYPE_5) == B_FALSE)
                    errout();
                break;
            case 28:
                if (bcSetGenCodeEx (fd, TC_NASA, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 29:
                if (bcSetGenCodeEx (fd, TC_XR3, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 30:
                if (bcSetGenCodeEx (fd, TC_2137, TC_SUBTYPE_NONE) == B_FALSE)
                    errout();
                break;
            case 0:
            default:
                tc_out = 0;
                break;
        }
    } while (tc_out!=0);
}

//****************************************************************************
// Description : Read from register
//****************************************************************************
void pci_read_register ()
{
    struct reg_rdwr_t rdwr;
    uint32_t iOffset;

    printf ("\n\n");
    printf ("Enter hex offset (0x00 - 0x80): 0x");
    scanf ("%x", &iOffset);
    rdwr.offset = (uint16_t)iOffset;

    if ( ioctl (fd, RDREGOFFSET, &rdwr) < 0 )
    {
        errout ();
    }
    else
    {
        printf ("Read Register 0x%02x: 0x%08x\n",
                (uint32_t)rdwr.offset, rdwr.data);
    }

    nxt();
}

//****************************************************************************
// Description : Write to register
//****************************************************************************
void pci_write_register ()
{
    struct reg_rdwr_t rdwr;
    uint32_t iOffset, iData;

    printf ("\n\n");
    printf ("Enter hex offset (0x00 - 0x80): 0x");
    scanf ("%x", &iOffset);
    printf ("Enter hex data: 0x");
    scanf ("%x", &iData);
    rdwr.offset = (uint16_t)iOffset;
    rdwr.data = iData;

    if ( ioctl (fd, WRREGOFFSET, &rdwr) < 0 )
    {
        errout ();
    }
    else
    {
        printf ("Wrote Register 0x%02x with 0x%08x\n",
                (uint32_t)rdwr.offset, (uint32_t)rdwr.data);
    }

    nxt();
}

//**************************************************************************************
// Description : Set events settings --- special for event2 and event3 support
//**************************************************************************************
void pci_sel_events_settings ()
{
    struct EventsData EvDat;
    int iVal;

    printf ("\n\n");
    printf ("Event trigger source (0 - event, 1 - heartbeat): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event trigger source.\n");
        nxt();
        return;
    }
    EvDat.evtsrc = (u_char)iVal;

    // Select event (or event1) settings
    printf ("Event enable (1 - enable, 0 - disable): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event enable.\n");
        nxt();
        return;
    }
    EvDat.evtctl = (u_char)iVal;

    printf ("Select event sense (0 - rising, 1 - falling): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event sense.\n");
        nxt();
        return;
    }
    EvDat.evtsense = (u_char)iVal;

    printf ("Select event capture lock (1 - lock, 0 - not lock): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event capture lock.\n");
        nxt();
        return;
    }
    EvDat.evtlock = (u_char)iVal;

    // Select event2 settings
    printf ("Event2 enable (1 - enable, 0 - disable): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event2 enable.\n");
        nxt();
        return;
    }
    EvDat.evt2ctl = (u_char)iVal;

    printf ("Select event2 sense (0 - rising, 1 - falling): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event2 sense.\n");
        nxt();
        return;
    }
    EvDat.evt2sense = (u_char)iVal;

    printf ("Select event2 capture lock (1 - lock, 0 - not lock): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event2 capture lock.\n");
        nxt();
        return;
    }
    EvDat.evt2lock = (u_char)iVal;

    // Select event3 settings
    printf ("Event3 enable (1 - enable, 0 - disable): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event3 enable.\n");
        nxt();
        return;
    }
    EvDat.evt3ctl = (u_char)iVal;

    printf ("Select event3 sense (0 - rising, 1 - falling): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event3 sense.\n");
        nxt();
        return;
    }
    EvDat.evt3sense = (u_char)iVal;

    printf ("Select event3 capture lock (1 - lock, 0 - not lock): ");
    if (scanf("%d", &iVal) != 1 && iVal != 0 && iVal != 1)
    {
        printf ("Error: invalid input for event3 capture lock.\n");
        nxt();
        return;
    }
    EvDat.evt3lock = (u_char)iVal;

    if (bcSetEventsData (fd, &EvDat) == B_TRUE)
    {
        printf ("Done selecting events settings.\n");
        nxt();
    }
    else
        errin();
}

//****************************************************************************
// Description : Advanced Menu
//****************************************************************************
void advanced_main ()
{
    int adv_menu;

    printf ("\n\n\n\nAdvanced Menu\n\n");
    printf (" 1. Control Jam-Sync\n");
    printf (" 2. Force Jam-Sync\n");
    printf (" 3. Set Clock Value\n");
    printf (" 4. Load D/A Converter\n");
    printf (" 5. Set Disciplining Gain\n");
    printf (" 6. Disconnect Battery to RTC Chip\n");
    printf (" 0. Back to main menu \n\n");
    printf ("    Select: ");
    scanf ("%d", &adv_menu);

    switch (adv_menu){
        case 1: pci_ctl_jam_sync();
            break;
        case 2: pci_frc_jam_sync();
            break;
        case 3: pci_set_clk_val();
            break;
        case 4: pci_set_da_con();
            break;
        case 5: pci_set_gain();
            break;
        case 6: pci_dis_rtc();
            break;
        case 0:
            break;
        default: usererr();
            break;
    }
}

//****************************************************************************
// Description : GPS Menu
//****************************************************************************
void gps_menu ()
{
    int gps_sel;

    printf ("\n\n\n\nGPS Packet Menu\n\n");
    printf (" 1. Request Packet 41 - Gps Time Packet\n");
    printf (" 2. Request Packet 42 - Gps Position Packet\n");
    printf (" 3. Request Packet 46 - Gps Health Packet\n");
    printf (" 0. Back to main menu \n\n");
    printf ("    Select: ");
    scanf ("%d", &gps_sel);

    switch (gps_sel){
        case 1: getPac41();
            break;
        case 2: getPac42();
            break;
        case 3: getPac46();
            break;
        case 0:
            break;
        default: usererr();
            break;
    }
}

//****************************************************************************
// Description : Interrupts Menu
//****************************************************************************
void interrupts_menu ()
{
    int int_menu = 0;

    do {
        printf ("\n\n\n\nInterrupts Menu\n\n");
        printf (" 1. Set Interrupts to start Interrupt Routine\n");
        printf (" 2. Stop Interrupt Routine\n");
        printf (" 3. Request Interrupts\n");
        printf (" 0. Back to main menu \n\n");
        printf ("    Select: ");
        scanf ("%d", &int_menu);

        switch (int_menu) {
            case 1: pci_set_ints ();
                    break;
            case 2: pci_stop_ints ();
                    break;
            case 3: pci_req_ints();
                    break;
            case 0:
            default: int_menu = 0;
                    break;
        }
    } while (int_menu!=0);
}

//****************************************************************************
// Description : DDS Menu
//****************************************************************************
void dds_menu ()
{
    int dds_sel;

    do {
        printf ("\n\n\n\nDDS Menu\n\n");
        printf (" 1. Set DDS Frequency\n");
        printf (" 2. Select Periodic or DDS Output\n");
        printf (" 3. Enable/Disable Periodic/DDS Output\n");
        printf (" 4. Set DDS Divider\n");
        printf (" 5. Set DDS Divider Source\n");
        printf (" 6. Set DDS Sync Mode\n");
        printf (" 7. Set DDS Multiplier\n");
        printf (" 8. Set DDS Period Value\n");
        printf (" 9. Set DDS Tuning Word\n");
        printf (" 0. Back to main menu \n\n");
        printf ("    Select: ");
        scanf ("%d", &dds_sel);

        switch (dds_sel)
        {
            case 1: dds_set_dds_frequency ();
                    break;
            case 2: dds_set_periodic_dds_select ();
                    break;
            case 3: dds_set_periodic_dds_enable ();
                    break;
            case 4: dds_set_dds_divider ();
                    break;
            case 5: dds_set_dds_divider_source ();
                    break;
            case 6: dds_set_dds_sync_mode ();
                    break;
            case 7: dds_set_dds_multiplier ();
                    break;
            case 8: dds_set_dds_period_value ();
                    break;
            case 9: dds_set_dds_tuning_word ();
                    break;
            case 0:
            default: dds_sel = 0;
                    break;
        }
    } while (dds_sel!=0);
}

//****************************************************************************
// Description : Main Menu
//****************************************************************************
int menu_main ()
{
    int menu;
    do
    {
        printf ("\n\n\n--------------------------------------------------------------------------------\n");
        printf ("                             Symmetricom - TT & M\n");
        printf ("                         bc635/637PCIe-V2 Configurator               Version 3.0\n");
        printf ("--------------------------------------------------------------------------------\n");
        printf ("  1. Read Current Time (Press enter to return to menu)\n");
        printf ("  2. Read Event Time   (Press enter to return to menu)\n"); 
        printf ("  3. Set Current Time \n");
        printf ("  4. Set Current Year\n");
        printf ("  5. Set Strobe Time\n");
        printf ("  6. Program Control Register\n");
        printf ("  7. Program Leap Event Seconds\n");
        printf ("  8. Select Time Format\n");
        printf ("  9. Select Operational Mode\n");
        printf (" 10. Select Decoding TimeCode Format\n");
        printf (" 11. Select TimeCode Output Format\n");
        printf (" 12. Select Clock Source\n");
        printf (" 13. Select Output Frequency\n");
        printf (" 14. Program Heartbeat counters\n");
        printf (" 15. Set Local Time Offset         \t\t16. Set Generator Time Offset\n");
        printf (" 17. Set Propagation Delay         \t\t18. Set Local Time Offset Flag\n");
        printf (" 19. Set Year Auto Increment Flag  \t\t20. Sync RTC to External Time Data\n");
        printf (" 21. Set GPS Time Format           \t\t22. Set GPS Mode Flag\n");
        printf (" 23. Software Reset                \t\t24. Request Time Settings\n");
        printf (" 25. Request Clock Settings        \t\t26. Request Offset Settings\n");
        printf (" 27. Request UTC Information       \t\t28. Request Model Information\n");
        printf (" 29. Request Firmware Version      \t\t30. Advanced Menu\n");
        printf (" 31. Interrupts Menu               \t\t32. GPS Packets Menu - bc637PCI Only\n");
        printf (" 33. PCI Revision ID               \t\t34. Read from Dual Port RAM\n");
        printf (" 35. Write to Dual Port RAM        \t\t36. Write ACK - Send DP Command\n");
        printf (" 37. DDS Menu - V2 Hardware Only\n");
        printf (" 38. Select TimeCode Input Format - V2 Hardware\n");
        printf (" 39. Select TimeCode Output Format - V2 Hardware\n");
        printf (" 40. Read from Register            \t\t41. Write to Register\n");
        printf (" 42. Select events settings\n");
        printf (" 43. Read Event2 Time  (Press enter to return to menu)\n");
        printf (" 44. Read Event3 Time  (Press enter to return to menu)\n");
        printf (" 0.  Exit the Program......................................................\n\n");
        printf ("     Select: ");
        scanf ("%d", &menu);

        switch(menu) 
        {
            case 1 : pci_read_time();
                break;
            case 2 : pci_read_event_time();
                break;
            case 3 : pci_set_time ();
                break;
            case 4 : pci_set_year();
                break;
            case 5 : pci_set_strobe();
                break;
            case 6 : pci_set_control();
                break;
            case 7 : pci_set_leap_sec ();
                break;
            case 8 : pci_time_format();
                break;
            case 9 : pci_mode();
                break;
            case 10 : pci_time_code();
                break;
            case 11 : pci_tc_out_format(); 
                break;
            case 12 : pci_set_clock_src();
                break;
            case 13 : pci_out_freq(); 
                break;
            case 14: pci_heartbeat();
                break;
            case 15: pci_set_loc_off ();
                break;
            case 16: pci_set_gen_off ();
                break;
            case 17: pci_set_prop_delay ();
                break;
            case 18 : pci_set_local_off_flg();
                break;
            case 19 : pci_set_yr_auto_inc_flg();
                break;
            case 20 : pci_sync_rtc();
                break;
            case 21 : pci_set_gps_tm_fmt();
                break;
            case 22 : pci_set_gps_mode_flg();
                break;
            case 23 : pci_sw_rest ();
                break;
            case 24 : pci_req_time_settings();
                break;
            case 25 : pci_req_clock_settings ();
                break;
            case 26 : pci_req_offsets_settings ();
                break;
            case 27 : pci_req_utc_info ();
                break;
            case 28 : pci_req_assembly ();
                break;
            case 29 : pci_req_fw_ver ();
                break;
            case 30 : advanced_main ();
                break;
            case 31 : interrupts_menu ();
                break;
            case 32 : gps_menu ();
                break;
            case 33 : pci_req_rev_id ();
                break;
            case 34 : pci_read_dp_ram ();
                break;
            case 35 : pci_write_dp_ram ();
                break;
            case 36 : pci_write_ack ();
                break;
            case 37 : dds_menu ();
                break;
            case 38 : pci_time_code_ex ();
                break;
            case 39 : pci_tc_out_format_ex ();
                break;
            case 40 : pci_read_register ();
                break;
            case 41 : pci_write_register ();
                break;
            case 42 : pci_sel_events_settings ();
                break;
            case 43 : pci_read_event2_time ();
                break;
            case 44 : pci_read_event3_time ();
                break;
            case 0 :
            default : menu = 0;
                printf("\n\nexiting..............................\n\n");
                break;
        }
    }while (menu);
    return 0;
}

//****************************************************************************
// Description : GPS Packet 41 - GPS Time Packet
//****************************************************************************
void getPac41 ()
{
    int i;

    printf ("\n\nGPS PACKET 41 - GPS TIME PACKET\n\n");
    write (fd, req41, sizeof (req41));
    read (fd, rbuf, STFP_MAX_READ);
	
    /* skip the length byte */
    for (i=0; i<78; i++)
        datbuf[i] = rbuf[i+1];
	
    /* skip the id byte and show data only */
    printf ("Raw Data: ");
    for (i = 1; i < 11; i++)
        printf ("%02X ", datbuf[i] & 0xff);
	
    printPac41 ();
    nxt();
}

void printPac41 ()
{
    u_char *pData = (u_char *)datbuf;
    pac41.id = getByte (&pData);
    pac41.time = getFloat (&pData);
    pac41.week = getShort (&pData);
    pac41.offset = getFloat (&pData);
    printf ("\nID:\t%02X\n", pac41.id);
    printf ("Time:\t%0.2f\n", pac41.time);
    printf ("Week:\t%hd\n", pac41.week);
    printf ("Offset:\t%0.2f\n\n", pac41.offset);
}

//****************************************************************************
// Description : GPS Packet 42 - Position Packet
//****************************************************************************
void getPac42 ()
{
    int i;

    printf ("\n\nGPS PACKET 42 - GPS POSITION PACKET\n\n");
    ioctl (fd, REQGPSPACKET, 0x42);
    read (fd, rbuf, STFP_MAX_READ);
	
    /* skip the length byte */
    for (i=0; i<78; i++)
        datbuf[i] = rbuf[i+1];

    /* skip the id byte and show data only */
    printf ("Raw Data: ");
    for (i = 1; i < 17; i++)
        printf ("%02X ", datbuf[i] & 0xff);

    printPac42 ();
    nxt();
}

void printPac42 ()
{
    u_char *pData = (u_char *)datbuf;
    pac42.id = getByte (&pData);
    pac42.X = getFloat (&pData);
    pac42.Y = getFloat (&pData);
    pac42.Z = getFloat (&pData);
    pac42.TofF = getFloat (&pData);
    printf ("\nID:\t\t%02X\n", pac42.id);
    printf ("X Coordinate:\t%0.2f\n", pac42.X);
    printf ("Y Coordinate:\t%0.2f\n", pac42.Y);
    printf ("Z Coordinate:\t%0.2f\n", pac42.Z);
    printf ("Time-of-fix: \t%0.2f\n\n", pac42.TofF);
}

//****************************************************************************
// Description : GPS Packet 46 - Health Packet
//****************************************************************************
void getPac46 ()
{
    int i;
	
    printf ("\n\nGPS PACKET 46 - GPS HEALTH PACKET\n\n");
    ioctl (fd, REQGPSPACKET, 0x46);
    read (fd, rbuf, STFP_MAX_READ);
	
    printf ("Raw Data: ");
    for (i = 0; i < 18; i++)
        printf ("%02X ", rbuf[i] & 0xff);

    printf ("\nID: \t%02X \nStatus: 0x%02X \nError: \t0x%02X\n",
        rbuf[1] & 0xff, rbuf[2] & 0xff, rbuf[3] & 0xff);
    nxt();
}

//****************************************************************************
// Description : Data formating 
//****************************************************************************
u_char getByte (u_char **ppData)
{
    return (*(*ppData)++);
}

short getShort (u_char **ppData)
{
    union { short s; u_char uc[2]; } conv;

    conv.uc[0] = *(*ppData)++;
    conv.uc[1] = *(*ppData)++;
    return (conv.s);
}

float getFloat (u_char **ppData)
{
    union { float f; u_char uc[4]; } conv;
    int i;

    for (i = 0; i < 4; i++)
        conv.uc[i] = *(*ppData)++;
    return (conv.f);
}

double getDouble (u_char **ppData)
{
    union { double f; u_char uc[8]; } conv;
    int i;

    for (i = 0; i < 8; i++)
        conv.uc[i] = *(*ppData)++;
    return (conv.f);
}

//****************************************************************************
// Description :  polls stdin to see if there is any input available
//****************************************************************************
int kbhit ()
{
   fd_set rfds;
   struct timeval tv;
   FD_ZERO (&rfds);
   FD_SET (0, &rfds);
   tv.tv_sec  = 0;
   tv.tv_usec = 0;
   select (1, &rfds, NULL, NULL, &tv);
   if (FD_ISSET(0, &rfds))
      return 1;
   return 0;
}

//****************************************************************************
// Description :  waits for selected time for any input to come into stdin then exits
// a value of -1 for seconds means wait forever
//****************************************************************************
int WaitForKbhit(int seconds, int mseconds)
{
   fd_set rfds;
   struct timeval tv;
   while (1)
   {
      FD_ZERO (&rfds);
      FD_SET (0, &rfds);
      tv.tv_sec  = seconds == -1 ? 60 : seconds;
      tv.tv_usec = mseconds;
      select (1, &rfds, NULL, NULL, &tv);
      if (FD_ISSET(0, &rfds))
         return 1;
      if (seconds != -1)
         break;
   }
   return 0;
}

//****************************************************************************
// Description : Error
//****************************************************************************
void usererr ()
{
    printf ("\nError: Unrecognized Selection!!!\n");
    getchar();
    printf("\n\n>>>>>>>>>>>>>>>>  Press Enter to continue  <<<<<<<<<<<<<<<<<\n");
    getchar(); 
}

void errout ()
{
    printf("\n\nError: getting data!!!!!!!!!!!!!!!!!!!!!!\n\n");
}

void errin ()
{
    printf("\n\nError: sending data!!!!!!!!!!!!!!!!!!!!!!\n\n");
}

void nxt ()
{
    getchar();
    printf("\n\n>>>>>>>>>>>>>>>>  Press Enter to continue  <<<<<<<<<<<<<<<<<\n");
    getchar(); 
}

//****************************************************************************
// API function: bcReqRevisionID
//****************************************************************************
boolean_t bcReqRevisionID (int fd, int* id)
{
    static int rev_id = -1;
    if (rev_id == -1)
    {
        if ( ioctl (fd, REQREVID, &rev_id) < 0 )
            return B_FALSE;
    }
    *id = rev_id;
    return B_TRUE;
}

//****************************************************************************
// API function: bcSetPeriodicDDSSelect
//****************************************************************************
boolean_t bcSetPeriodicDDSSelect (int fd, int sel)
{
    struct dds_command_t ddscmd;
    int revID;

    if (sel != SELECT_PERIODIC_OUT && sel != SELECT_DDS_OUT)
        return B_FALSE;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    ddscmd.cmd_id = COM_SET_PRD_DDS_SEL;
    ddscmd.data.cmd_byte[0] = (u_char)(sel & 0xFF);
    ddscmd.cmd_len = 1;

    if ( ioctl (fd, DDSCOMMAND, &ddscmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetPeriodicDDSEnable
//****************************************************************************
boolean_t bcSetPeriodicDDSEnable (int fd, int enable)
{
    struct dds_command_t ddscmd;
    int revID;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    ddscmd.cmd_id = COM_SET_PRD_DDS_ENA;
    ddscmd.data.cmd_byte[0] = enable ? 1 : 0;
    ddscmd.cmd_len = 1;

    if ( ioctl (fd, DDSCOMMAND, &ddscmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetDDSDivider
//****************************************************************************
boolean_t bcSetDDSDivider (int fd, int divider)
{
    struct dds_command_t ddscmd;
    int revID;

    if (divider != DDS_DIVIDE_BY_1E0 &&
        divider != DDS_DIVIDE_BY_1E1 &&
        divider != DDS_DIVIDE_BY_1E2 &&
        divider != DDS_DIVIDE_BY_1E3 &&
        divider != DDS_DIVIDE_BY_1E4 &&
        divider != DDS_DIVIDE_BY_1E5 &&
        divider != DDS_DIVIDE_BY_1E6 &&
        divider != DDS_DIVIDE_BY_1E7 &&
        divider != DDS_DIVIDE_BY_PREG)
        return B_FALSE;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    ddscmd.cmd_id = COM_SET_DDS_DIVIDER;
    ddscmd.data.cmd_byte[0] = (u_char)(divider & 0xFF);
    ddscmd.cmd_len = 1;

    if ( ioctl (fd, DDSCOMMAND, &ddscmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetDDSDividerSource
//****************************************************************************
boolean_t bcSetDDSDividerSource (int fd, int src)
{
    struct dds_command_t ddscmd;
    int revID;

    if (src != DDS_DIVIDER_SRC_DDS &&
        src != DDS_DIVIDER_SRC_MULT &&
        src != DDS_DIVIDER_SRC_100MHZ)
        return B_FALSE;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    ddscmd.cmd_id = COM_SET_DDS_DIV_SRC;
    ddscmd.data.cmd_byte[0] = (u_char)(src & 0xFF);
    ddscmd.cmd_len = 1;

    if ( ioctl (fd, DDSCOMMAND, &ddscmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetDDSSyncMode
//****************************************************************************
boolean_t bcSetDDSSyncMode (int fd, int mode)
{
    struct dds_command_t ddscmd;
    int revID;

    if (mode != DDS_SYNC_MODE_FRAC && mode != DDS_SYNC_MODE_CONT)
        return B_FALSE;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    ddscmd.cmd_id = COM_SET_DDS_DIV_SYNC;
    ddscmd.data.cmd_byte[0] = (u_char)(mode & 0xFF);
    ddscmd.cmd_len = 1;

    if ( ioctl (fd, DDSCOMMAND, &ddscmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetDDSMultiplier
//****************************************************************************
boolean_t bcSetDDSMultiplier (int fd, int mult)
{
    struct dds_command_t ddscmd;
    int revID;

    if (mult != DDS_MULTIPLY_BY_1 &&
        mult != DDS_MULTIPLY_BY_2 &&
        mult != DDS_MULTIPLY_BY_3 &&
        mult != DDS_MULTIPLY_BY_4 &&
        mult != DDS_MULTIPLY_BY_6 &&
        mult != DDS_MULTIPLY_BY_8 &&
        mult != DDS_MULTIPLY_BY_10 &&
        mult != DDS_MULTIPLY_BY_16)
        return B_FALSE;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    ddscmd.cmd_id = COM_SET_DDS_MULTI;
    ddscmd.data.cmd_byte[0] = (u_char)(mult & 0xFF);
    ddscmd.cmd_len = 1;

    if ( ioctl (fd, DDSCOMMAND, &ddscmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetDDSPeriodValue
//****************************************************************************
boolean_t bcSetDDSPeriodValue (int fd, uint32_t period)
{
    struct dds_command_t ddscmd;
    int revID;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    ddscmd.cmd_id = COM_SET_DDS_PERIOD;
    ddscmd.data.cmd_number = period;
    ddscmd.cmd_len = 4;

    if ( ioctl (fd, DDSCOMMAND, &ddscmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetDDSTuningWord
//****************************************************************************
boolean_t bcSetDDSTuningWord (int fd, uint32_t tuneWord)
{
    struct dds_command_t ddscmd;
    int revID;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    ddscmd.cmd_id = COM_SET_DDS_TUNEWORD;
    ddscmd.data.cmd_number = tuneWord;
    ddscmd.cmd_len = 4;

    if ( ioctl (fd, DDSCOMMAND, &ddscmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetDDSFrequency
//****************************************************************************
boolean_t bcSetDDSFrequency (int fd, double fFreq)
{
    int revID, divider, mult;
    double dDDSFreq;
    uint32_t tuneWord;

    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
        return B_FALSE;

    if (fFreq < 1)
    {
        dDDSFreq = fFreq * 1E7;
        divider = DDS_DIVIDE_BY_1E7;
    }
    else if (fFreq < 1E1)
    {
        dDDSFreq = fFreq * 1E6;
        divider = DDS_DIVIDE_BY_1E6;
    }
    else if (fFreq < 1E2)
    {
        dDDSFreq = fFreq * 1E5;
        divider = DDS_DIVIDE_BY_1E5;
    }
    else if (fFreq < 1E3)
    {
        dDDSFreq = fFreq * 1E4;
        divider = DDS_DIVIDE_BY_1E4;
    }
    else if (fFreq < 1E4)
    {
        dDDSFreq = fFreq * 1E3;
        divider = DDS_DIVIDE_BY_1E3;
    }
    else if (fFreq < 1E5)
    {
        dDDSFreq = fFreq * 1E2;
        divider = DDS_DIVIDE_BY_1E2;
    }
    else if (fFreq < 1E6)
    {
        dDDSFreq = fFreq * 1E1;
        divider = DDS_DIVIDE_BY_1E1;
    }
    else
    {
        dDDSFreq = fFreq;
        divider = DDS_DIVIDE_BY_1E0;
    }

    if (dDDSFreq > 88E6)
        mult = 8;
    else if (dDDSFreq > 44E6)
        mult = 4;
    else if (dDDSFreq > 22E6)
        mult = 2;
    else
        mult = 1;

    tuneWord = (uint32_t)(dDDSFreq * (32/mult));

#undef __DEBUG__
#ifdef __DEBUG__
    printf(">> Input frequency: %f, DDSFrequency: %f\n", fFreq, dDDSFreq);
    printf("-- Divider: %d, Multiplier: %d, TuneWord: %d\n", divider, mult, tuneWord);
#endif

    if (bcSetPeriodicDDSSelect (fd, SELECT_DDS_OUT) == B_TRUE &&
        bcSetPeriodicDDSEnable (fd, (int)B_TRUE) == B_TRUE &&
        bcSetDDSTuningWord (fd, tuneWord) ==  B_TRUE &&
        bcSetDDSDivider (fd, divider) == B_TRUE &&
        bcSetDDSMultiplier (fd, mult) == B_TRUE &&
        bcSetDDSSyncMode (fd, DDS_SYNC_MODE_FRAC) == B_TRUE)
    {
        int src;
        src = (mult > 1 ? DDS_DIVIDER_SRC_MULT : DDS_DIVIDER_SRC_DDS);
        return bcSetDDSDividerSource (fd, src);
    }

    return B_FALSE;
}

//****************************************************************************
// API function: bcSetTcInEx
//****************************************************************************
boolean_t bcSetTcInEx (int fd, uchar_t TcIn, uchar_t SubType)
{
    int revID;
    struct tc_command_t tccmd;
    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
    {
        // This is V1 hardware. It does not support sub type.
        if (SubType != TC_SUBTYPE_NONE)
            return B_FALSE;

        if (TcIn != TC_IRIGA && TcIn != TC_IRIGB &&
            TcIn != TC_IEEE && TcIn != TC_NASA)
            return B_FALSE;
    }
    else
    {
        // Here comes the V2 hardware.
        switch (TcIn)
        {
            case TC_IEEE:
            case TC_NASA:
            case TC_2137:
            case TC_XR3:
                if (SubType != TC_SUBTYPE_NONE)
                    return B_FALSE;
                break;

            case TC_IRIGA:
            case TC_IRIGE:
            case TC_IRIGe:
            case TC_IRIGG:
                if (SubType != TC_SUBTYPE_NONE &&
                    SubType != TC_SUBTYPE_Y)
                    return B_FALSE;
                break;

            case TC_IRIGB:
                if (SubType != TC_SUBTYPE_NONE &&
                    SubType != TC_SUBTYPE_Y &&
                    SubType != TC_SUBTYPE_T)
                    return B_FALSE;
                break;

            default:
                return B_FALSE;
        }
    }
    // Make driver call
    tccmd.cmd_id = COM_SETTCFORMAT;
    tccmd.type = TcIn;
    tccmd.subtype = SubType;
    if ( ioctl (fd, TCFORMATEX, &tccmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcSetGenCodeEx
//****************************************************************************
boolean_t bcSetGenCodeEx (int fd, uchar_t GenTc, uchar_t SubType)
{
    int revID;
    struct tc_command_t tccmd;
    bcReqRevisionID (fd, &revID);
    if (revID < BC_PCI_V2_REV_ID_START || revID > BC_PCI_V2_REV_ID_END)
    {
        // This is V1 hardware. It does not support sub type.
        if (SubType != TC_SUBTYPE_NONE)
            return B_FALSE;

        if (GenTc != TC_IRIGB && GenTc != TC_IEEE)
            return B_FALSE;
    }
    else
    {
        // Here comes the V2 hardware.
        switch (GenTc)
        {
            case TC_IEEE:
            case TC_NASA:
            case TC_2137:
            case TC_XR3:
                if (SubType != TC_SUBTYPE_NONE)
                    return B_FALSE;
                break;

            case TC_IRIGA:
                if (SubType != TC_SUBTYPE_0 &&
                    SubType != TC_SUBTYPE_1 &&
                    SubType != TC_SUBTYPE_2 &&
                    SubType != TC_SUBTYPE_3 &&
                    SubType != TC_SUBTYPE_4 &&
                    SubType != TC_SUBTYPE_5 &&
                    SubType != TC_SUBTYPE_6 &&
                    SubType != TC_SUBTYPE_7)
                    return B_FALSE;
                break;

            case TC_IRIGB:
                if (SubType != TC_SUBTYPE_NONE &&
                    SubType != TC_SUBTYPE_0 &&
                    SubType != TC_SUBTYPE_1 &&
                    SubType != TC_SUBTYPE_2 &&
                    SubType != TC_SUBTYPE_3 &&
                    SubType != TC_SUBTYPE_4 &&
                    SubType != TC_SUBTYPE_5 &&
                    SubType != TC_SUBTYPE_6 &&
                    SubType != TC_SUBTYPE_7 &&
                    SubType != TC_SUBTYPE_T)
                    return B_FALSE;
                break;

            case TC_IRIGE:
            case TC_IRIGe:
                if (SubType != TC_SUBTYPE_1 &&
                    SubType != TC_SUBTYPE_2 &&
                    SubType != TC_SUBTYPE_5 &&
                    SubType != TC_SUBTYPE_6)
                    return B_FALSE;
                break;

            case TC_IRIGG:
                if (SubType != TC_SUBTYPE_5)
                    return B_FALSE;
                break;

            default:
                return B_FALSE;
        }
    }
    // Make driver call
    tccmd.cmd_id = COM_SETTCOUTFMT;
    tccmd.type = GenTc;
    tccmd.subtype = SubType;
    if ( ioctl (fd, TCFORMATEX, &tccmd) < 0 )
        return B_FALSE;

    return B_TRUE;
}

//****************************************************************************
// API function: bcReqEventsData
//****************************************************************************
boolean_t bcReqEventsData (int fd, struct EventsData *pDat)
{
    unsigned long ctlreg;

    if (ioctl (fd, RDCONTROL, &ctlreg) < 0)
        return B_FALSE;

    pDat->evtsrc = (u_char)(ctlreg & 0x00000002 ? 1 : 0);

    pDat->evtctl   = (u_char)(ctlreg & 0x00000008 ? 1 : 0);
    pDat->evtsense = (u_char)(ctlreg & 0x00000004 ? 1 : 0);
    pDat->evtlock  = (u_char)ctlreg & 0x01;

    // Event2 ctl, sense, lock
    pDat->evt2ctl   = (u_char)(ctlreg & 0x00000400 ? 1 : 0);
    pDat->evt2sense = (u_char)(ctlreg & 0x00000200 ? 1 : 0);
    pDat->evt2lock  = (u_char)(ctlreg & 0x00000100 ? 1 : 0);

    // Event3 ctl, sense, lock
    pDat->evt3ctl   = (u_char)(ctlreg & 0x00004000 ? 1 : 0);
    pDat->evt3sense = (u_char)(ctlreg & 0x00002000 ? 1 : 0);
    pDat->evt3lock  = (u_char)(ctlreg & 0x00001000 ? 1 : 0);
    return B_TRUE;
}

//****************************************************************************
// API function: bcSetEventsData
//****************************************************************************
boolean_t bcSetEventsData (int fd, struct EventsData *pDat)
{
    unsigned long ctlreg;

    if (ioctl (fd, RDCONTROL, &ctlreg) < 0)
        return B_FALSE;

    if (pDat->evtsrc)
        ctlreg |= 0x00000002;
    else
        ctlreg &= ~0x00000002;

    if (pDat->evtctl)
        ctlreg |= 0x00000008;
    else
        ctlreg &= ~0x00000008;

    if (pDat->evtsense)
        ctlreg |= 0x00000004;
    else
        ctlreg &= ~0x00000004;

    if (pDat->evtlock)
        ctlreg |= 0x00000001;
    else
        ctlreg &= ~0x00000001;

    // Event2 ctl, sense, lock
    if (pDat->evt2ctl)
        ctlreg |= 0x00000400;
    else
        ctlreg &= ~0x00000400;

    if (pDat->evt2sense)
        ctlreg |= 0x00000200;
    else
        ctlreg &= ~0x00000200;

    if (pDat->evt2lock)
        ctlreg |= 0x00000100;
    else
        ctlreg &= ~0x00000100;

    // Event3 ctl, sense, lock
    if (pDat->evt3ctl)
        ctlreg |= 0x00004000;
    else
        ctlreg &= ~0x00004000;

    if (pDat->evt3sense)
        ctlreg |= 0x00002000;
    else
        ctlreg &= ~0x00002000;

    if (pDat->evt3lock)
        ctlreg |= 0x00001000;
    else
        ctlreg &= ~0x00001000;

    if (ioctl (fd, SETCONTROL, ctlreg) < 0)
        return B_FALSE;

    return B_TRUE;
}
